/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang.coverage;

import jatva.lang.RuntimeSpecificationMethodException;
import jatva.util.TwoBitSet;

/**
 * The class for analysis of disjunct-based coverages.
 *
 * @author Victor Kuliamin
 */
public class DisjunctAnalyser
{
  private SEOperationInfo _info = null;
  private TwoBitSet _formulasSet = new TwoBitSet();

  public DisjunctAnalyser(SEOperationInfo info)
  {
    _info = info;
  }

  public boolean recordFormulaValue(int id, boolean value)
  {
    byte oldValue = _formulasSet.get(id);
    byte newValue = value?DisjunctInfo.True:DisjunctInfo.False;
    if(oldValue != DisjunctInfo.Undefined && oldValue != newValue)
    {
      throw new RuntimeSpecificationMethodException
        (
            "Prime formula has two different values in one situation"
          , new String[]{ "class", "method", "formula", "old value", "new value"}
          , new String[]{  _info.getDeclaringClass(), _info.getPrettySignature(), _info._formulas.get(id)
                         , (oldValue == DisjunctInfo.True)?"true":"false"
                         , "" + value
                        }
        );
    }

    _formulasSet.set(id, newValue);

    return value;
  }

  public int getPartitionElementIndex(String coverage)
  {
    int disjunct = -1;
    DisjunctsMap map = _info.getDisjunctsMap();

    DisjunctsMap.DisjunctMapEnumerator en = map.getEnumerator();

    while(en.moveNext())
    {
      DisjunctsMap.Entry e = en.current();
//        Chase.Trace.Tracer.TraceUserInfo("Formulas : " + e.value.Values);
      if(DisjunctInfo.compareValues(_formulasSet, e.value.getValues()))
//        if(_formulasSet.Equals(e.value.Values))
      {
        if(disjunct != -1)
        {
          throw new RuntimeSpecificationMethodException
            (
              "Several disjuncts match the situation"
              , new String[]{  "class", "method", "current situation"
                             , "disjunct 1", "disjunct 1 situation"
                             , "disjunct 2", "disjunct 2 situation"
                            }
              , new String[]{  _info.getDeclaringClass(), _info.getPrettySignature()
                             , DisjunctInfo.getString(_formulasSet)
                             , "" + disjunct, _info._disjuncts.get(disjunct).getFormulasValues()
                             , "" + e.key, e.value.getFormulasValues()
                            }
            );
        }
        else
          disjunct = e.key;
      }
    }

    if(disjunct == -1)
    {
//        Chase.Trace.Tracer.TraceUserInfo("Current formulas : " + _formulasSet);
      throw new RuntimeSpecificationMethodException
        (
            "No suitable disjuncts found for the situation"
          , new String[]{ "class", "method", "current situation" }
          , new String[]{ _info.getDeclaringClass(), _info.getPrettySignature(), DisjunctInfo.getString(_formulasSet) }
        );
    }

    if     (coverage == null) return 0;
    else if(coverage.equals(SEOperationInfo.BRANCHES))   return map.get(disjunct).getBranchId();
    else if(coverage.equals(SEOperationInfo.MARKS))      return map.get(disjunct).getMarkedPathId();
    else if(coverage.equals(SEOperationInfo.PREDICATES)) return map.get(disjunct).getPredicateId();
    else if(coverage.equals(SEOperationInfo.DISJUNCTS))  return disjunct;
    else                      return 0;
  }
}
